
package PPIx::Literal::Unknown;
$PPIx::Literal::Unknown::VERSION = '0.1.0';
# ABSTRACT: Representation of non-literal nodes

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

PPIx::Literal::Unknown - Representation of non-literal nodes

=head1 VERSION

version 0.1.0

=head1 AUTHOR

Adriano Ferreira <ferreira@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Adriano Ferreira.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
